#pragma once

#include "ChafKitCore/ChafKitCore.h"

template <typename T>
struct CHAFKIT_API TChafKitResult
{
    T Value;
    ChafKitCore::ChafKitResult ErrorCode;
    bool bSuccess;
    FString ErrorMessage;

    static TChafKitResult<T> Success(const T &InValue)
    {
        return TChafKitResult<T>{InValue, ChafKitCore::ChafKitResult::Ok, true, TEXT("Operation completed successfully.")};
    }

    static TChafKitResult<T> Failure(ChafKitCore::ChafKitResult InErrorCode)
    {
        return TChafKitResult<T>{T(), InErrorCode, false, GetErrorMessageFromResult(InErrorCode)};
    }

private:
    static FString GetErrorMessageFromResult(ChafKitCore::ChafKitResult result)
    {
        switch (result)
        {
        case ChafKitCore::ChafKitResult::InvalidParameters:
            return TEXT("One or more parameters are invalid.");
        case ChafKitCore::ChafKitResult::NotFound:
            return TEXT("The specified item could not be found.");
        default:
            return TEXT("An unknown error occurred.");
        }
    }
};
